from soaplib.wsgi_soap import SimpleWSGISoapApp
from soaplib.service import soapmethod
from soaplib.serializers.primitive import String, Integer, Array

'''
This is a simple HelloWorld example to show the basics of writing
a webservice using soaplib, starting a server, and creating a service
client. 
'''

class HelloWorldService(SimpleWSGISoapApp):
    
    @soapmethod(String,Integer,_returns=Array(String))
    def say_hello(self,name,times):
    	'''
        Docstrings for service methods appear as documentation in the wsdl
        <b>what fun</b>
	@param name the name to say hello to
	@param the number of times to say hello
	@return the completed array
    	'''
        results = []
        for i in range(0,times):
            results.append('Hello, %s'%name)
        return results

def make_client():
    from soaplib.client import make_service_client
    client = make_service_client('http://localhost:7889/',HelloWorldService())
    return client
    
if __name__=='__main__':
    try:
        from wsgiref.simple_server import make_server
        server = make_server('localhost', 7889, HelloWorldService())
        server.serve_forever()
    except ImportError:
        print "Error: example server code requires Python >= 2.5"



import unittest
import datetime
from soaplib.etimport import ElementTree as et

from soaplib.serializers.primitive import Integer, String, Repeating
from soaplib.serializers.clazz import ClassSerializer
from soaplib.serializers.primitive import String, Integer, DateTime, Float, Array
from soaplib.soap import Message, MethodDescriptor, make_soap_envelope, make_soap_fault, from_soap

class Address(ClassSerializer):
    class types:
        street = String
        city = String
        zip = Integer
        since = DateTime
        laditude = Float
        longitude = Float

class Person(ClassSerializer):
    class types:
        name = String
        birthdate = DateTime
        age = Integer
        addresses = Array(Address)
        titles = Array(String)


class test(unittest.TestCase):

    def test_simple_message(self):
        
        m = Message('myMessage',[('s',String),('i',Integer)])
        e = m.to_xml('a',43)

        self.assertEquals(e.tag,'myMessage')
        
        self.assertEquals(e.getchildren()[0].tag,'s')
        self.assertEquals(e.getchildren()[1].tag,'i')

        self.assertEquals(e.getchildren()[0].text,'a')
        self.assertEquals(e.getchildren()[1].text,'43')
        
        values = m.from_xml(e)
        
        self.assertEquals('a',values[0])
        self.assertEquals(43,values[1])
        
    def test_href(self):
        
        a = '''<?xml version="1.0" encoding="utf-8"?>
        <soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xmlns:xsd="http://www.w3.org/2001/XMLSchema"
        xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/"
        xmlns:tns="http://tempuri.org/"
        xmlns:types="http://example.com/encodedTypes"
        xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
          <soap:Body soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
            <tns:myResponse xsi:type="tns:myResponse">
              <myResult href="#id1" />
            </tns:myResponse>
            <soapenc:Array id="id1" soapenc:arrayType="tns:MyData[2]">
              <Item href="#id2" />
              <Item href="#id3" />
            </soapenc:Array>
            <tns:MyData id="id2" xsi:type="tns:MyData">
              <Machine xsi:type="xsd:string">somemachine</Machine>
              <UserName xsi:type="xsd:string">someuser</UserName>
            </tns:MyData>
            <tns:MyData id="id3" xsi:type="tns:MyData">
              <Machine xsi:type="xsd:string">machine2</Machine>
              <UserName xsi:type="xsd:string">user2</UserName>
            </tns:MyData>
          </soap:Body>
        </soap:Envelope>
        '''
        payload,header = from_soap(a)
        self.assertEquals(len(payload.getchildren()[0].getchildren()),2) # quick and dirty test href reconstruction
        
    def test_namespaces(self):
        m = Message('{some_namespace}myMessage',[('s',String),('i',Integer)])
        e = m.to_xml('a',43)
        self.assertEquals(e.tag,'{some_namespace}myMessage')

        m1 = Message('myMessage',[('{some_namespace}s',String),('i',Integer)],ns='some_namespace')
        e2 = m1.to_xml('a',43)
        self.assertEquals(e2.nsmap[None],'some_namespace')

    def test_class_to_xml(self):
        m = Message('myMessage',[('p',Person)])
        
        p = Person()
        p.name = 'steve-o'
        p.age = 2

        element = m.to_xml(p)

        self.assertEquals(element.tag,'myMessage')
        self.assertEquals(element.getchildren()[0].find('name').text,'steve-o')
        self.assertEquals(element.getchildren()[0].find('age').text,'2')
        self.assertEquals(len(element.getchildren()[0].find('addresses').getchildren()),0)
        
        p1 = m.from_xml(element)[0]
        
        self.assertEquals(p1.name,p.name)
        self.assertEquals(p1.age,p.age)
        self.assertEquals(p1.addresses,[])
        

    def test_to_xml_nested(self):
        
        m = Message('myMessage',[('p',Person)])
        
        p = Person()
        p.name = 'steve-o'
        p.age = 2
        p.addresses = []

        for i in range(0,1000):
            a = Address()
            a.street = '123 happy way'
            a.zip = i
            a.laditude = '45.22'
            a.longitude = '444.234'
            p.addresses.append(a)

        element = m.to_xml(p)

        self.assertEquals('myMessage',element.tag)
        addresses = element.getchildren()[0].find('addresses').getchildren()
        self.assertEquals(1000,len(addresses))
        self.assertEquals('0',addresses[0].find('zip').text)
        
    def test_soap_envelope(self):
        m = Message('myMessage',[('p',Person)])
        env = make_soap_envelope(m.to_xml(Person()))
        
        self.assertTrue(env.tag.endswith('Envelope'))
        self.assertTrue(env.getchildren()[0].tag.endswith('Body'))

        m = Message('myMessage',[('p',Person)])
        env = make_soap_envelope(m.to_xml(Person()), header_elements=[et.Element('header1'),et.Element('header2')])

        env = et.fromstring(et.tostring(env))
        
        self.assertTrue(env.getchildren()[0].tag.endswith('Header'))
        self.assertEquals(len(env.getchildren()[0].getchildren()),2)
        self.assertTrue(env.getchildren()[1].tag.endswith('Body'))
        
    def test_soap_fault(self):
        fault = make_soap_fault('something happened')
        fault = et.fromstring(et.tostring(fault))
        
        self.assertTrue(fault.getchildren()[0].tag.endswith,'Body')
        self.assertTrue(fault.getchildren()[0].getchildren()[0].tag.endswith('Fault'))
        f = fault.getchildren()[0].getchildren()[0]
        
        self.assertEquals(f.find('faultstring').text,'something happened')
        self.assertEquals(f.find('faultcode').text,'Server')
        self.assertEquals(f.find('detail').text,None)
        
        fault = make_soap_fault('something happened','DatabaseError','error on line 12')
        
        fault = et.fromstring(et.tostring(fault))
        
        f = fault.getchildren()[0].getchildren()[0]
        self.assertEquals(f.find('faultstring').text,'something happened')
        self.assertEquals(f.find('faultcode').text,'DatabaseError')
        self.assertEquals(f.find('detail').text,'error on line 12')
        
    def test_message_repeating(self):
        m = Message('myMessage',[('p',Repeating(String))])
        method = m.to_xml(["a","b","c","d"])
        self.assertEquals(len(method.getchildren()),4)

        data = m.from_xml(method)
    
        self.assertEquals(data,[["a","b","c","d"]])
        
    

def test_suite():
    loader = unittest.TestLoader()
    return loader.loadTestsFromTestCase(test)

if __name__== '__main__':
    unittest.TextTestRunner().run(test_suite())

                    


from soaplib.wsgi_soap import SimpleWSGISoapApp
from soaplib.service import soapmethod
from soaplib.serializers.primitive import String, Integer, Array
from soaplib.serializers.clazz import ClassSerializer

'''
This example shows how to define and use complex structures
in soaplib.  This example uses an extremely simple in-memory
dictionary to store the User objects.
'''

user_database = {}
userid_seq = 1

from soaplib.serializers.primitive import String, Integer, Array
from soaplib.serializers.clazz import ClassSerializer
class Permission(ClassSerializer):
    class types:
        application = String
        feature = String

class User(ClassSerializer):
    class types:
        userid = Integer
        username = String
        firstname = String
        lastname = String
        permissions = Array(Permission)

class UserManager(SimpleWSGISoapApp):
    
    @soapmethod(User,_returns=Integer)
    def add_user(self,user):
        global user_database
        global userid_seq
        user.userid = userid_seq
        userid_seq = userid_seq+1
        user_database[user.userid] = user
        return user.userid
        
    @soapmethod(Integer,_returns=User)
    def get_user(self,userid):
        global user_database
        return user_database[userid]
        
    @soapmethod(User)
    def modify_user(self,user):
        global user_database
        user_database[user.userid] = user
        
    @soapmethod(Integer)
    def delete_user(self,userid):
        global user_database
        del user_database[userid]
        
    @soapmethod(_returns=Array(User))
    def list_users(self):
        global user_database
        return [v for k,v in user_database.items()]

if __name__=='__main__':
    try:
        from wsgiref.simple_server import make_server
        server = make_server('localhost', 7789, UserManager())
        server.serve_forever()
    except ImportError:
        print "Error: example server code requires Python >= 2.5"



import unittest

import serializers
import client_test
import soap_test
import service_test

def test_suite():
    suite = serializers.test_suite()
    #suite.addTests(client_test.test_suite())
    suite.addTests(soap_test.test_suite())
    suite.addTests(service_test.test_suite())
    return suite
    
if __name__== '__main__':
    unittest.TextTestRunner().run(test_suite())


import unittest
import datetime
from soaplib.etimport import ElementTree

from soaplib.serializers.primitive import *
from soaplib.serializers.clazz import *
from soaplib.service import *
from soaplib.wsgi_soap import *

class Address(ClassSerializer):
    class types:
        street = String
        city = String
        zip = Integer
        since = DateTime
        laditude = Float
        longitude = Float

class Person(ClassSerializer):
    class types:
        name = String
        birthdate = DateTime
        age = Integer
        addresses = Array(Address)
        titles = Array(String)


class Request(ClassSerializer):
    class types:
        param1 = String
        param2 = Integer

class Response(ClassSerializer):
    class types:
        param1 = Float
        
class TestService(SoapServiceBase):

    @soapmethod(String,_returns=String)
    def aa(self,s):
        return s

    @soapmethod(String, Integer, _returns=DateTime)
    def a(self, s, i):
        return datetime.datetime.now()

    @soapmethod(Person, String, Address, _returns=Address)
    def b(self, p,s,a):
        return Address()

    @soapmethod(Person, isAsync=True)
    def d(self, Person):
        pass

    @soapmethod(Person, isCallback=True)
    def e(self, Person):
        pass
        
    @soapmethod(String, String, String, _returns=String, 
                _inputVariableNames={'_from':'from','_self':'self','_import':'import'},
                _outVariableName="return")
    def f(self,_from, _self, _import):
        return '1234'

class OverrideNamespaceService(SimpleWSGISoapApp):
    __tns__ = "http://someservice.com/override"
    
    @soapmethod(String,_returns=String)
    def mymethod(self,s):
        return s

class test(unittest.TestCase):
    '''
    Most of the service tests are excersized through the interop tests
    '''
    
    def setUp(self):
        self.service = TestService()
        self._wsdl = self.service.wsdl('')
        self.wsdl = ElementTree.fromstring(self._wsdl)
        
    def test_portypes(self):
        porttype = self.wsdl.find('{http://schemas.xmlsoap.org/wsdl/}portType')
        self.assertEquals(len(self.service._soap_methods),len(porttype.getchildren()))

    def test_override_default_names(self):
        wsdl = ElementTree.fromstring(OverrideNamespaceService().wsdl(''))
        self.assertEquals(wsdl.get('targetNamespace'),"http://someservice.com/override")
        
    def test_override_param_names(self):
        for n in ['self','import','return','from']:
            self.assertTrue(n in self._wsdl)
        

def test_suite():
    loader = unittest.TestLoader()
    return loader.loadTestsFromTestCase(test)

if __name__== '__main__':
    unittest.TextTestRunner().run(test_suite())




#!/usr/bin/env python

from soaplib.wsgi_soap import SimpleWSGISoapApp
from soaplib.service import soapmethod
from soaplib.serializers.primitive import String, Integer, Array
from soaplib.serializers.binary import Attachment

class HelloWorldService(SimpleWSGISoapApp):

    @soapmethod(Attachment,Integer,_returns=Array(String), _mtom=True)
    def say_hello(self,name,times):
        results = []
        for i in range(0,times):
            results.append('Hello, %s'%name.data)
        return results
        
if __name__=='__main__':
    try:
        from wsgiref.simple_server import make_server
        server = make_server('localhost', 7789, HelloWorldService())
        server.serve_forever()
    except ImportError:
        print "Error: example server code requires Python >= 2.5"



from soaplib.wsgi_soap import SimpleWSGISoapApp
from soaplib.service import soapmethod
from soaplib.serializers.primitive import String, Integer, Array
from soaplib.serializers.binary import Attachment


from tempfile import mkstemp
import os

class DocumentArchiver(SimpleWSGISoapApp):
    
    @soapmethod(Attachment,_returns=String)
    def archive_document(self,document):
        '''
        This method accepts an Attachment object, and returns the filename of the
        archived file
        '''
        fd,fname = mkstemp()
        os.close(fd)
        
        document.fileName = fname
        document.save_to_file()
        
        return fname

    @soapmethod(String,_returns=Attachment)
    def get_archived_document(self,file_path):
        '''
        This method loads a document from the specified file path
        and returns it.  If the path isn't found, an exception is
        raised.
        '''
        if not os.path.exists(file_path):
            raise Exception("File [%s] not found"%file_path)
        
        document = Attachment(fileName=file_path)
        # the service automatically loads the data from the file.
        # alternatively, The data could be manually loaded into memory
        # and loaded into the Attachment like:
        #   document = Attachment(data=data_from_file)
        return document
        

def make_client():
    from soaplib.client import make_service_client
    client = make_service_client('http://localhost:7889/',DocumentArchiver())
    return client
    
if __name__=='__main__':
    try:
        from wsgiref.simple_server import make_server
        server = make_server('localhost', 7889, DocumentArchiver())
        server.serve_forever()
    except ImportError:
        print "Error: example server code requires Python >= 2.5"


from soaplib.wsgi_soap import SimpleWSGISoapApp
from soaplib.service import soapmethod
from soaplib.serializers.primitive import String, Integer, Array
from soaplib.wsgi_soap import request

from time import time

'''
This example is an enhanced version of the HelloWorld example that
uses service 'hooks' to apply cross-cutting behavior to the service.
In this example, the service hooks are used to gather performance 
information on both the method execution as well as the duration 
of the entire call, including serialization and deserialization. The
available hooks are:

    * onCall
    * onWsdl
    * onWsdlException
    * onMethodExec
    * onResults
    * onException
    * onReturn
    
These method can be used to easily apply cross-cutting functionality
accross all methods in the service to do things like database transaction
management, logging and measuring performance.  This example also 
employs the threadlocal request (soaplib.wsgi_soap.request) object
to hold the data points for this request.
'''

class HelloWorldService(SimpleWSGISoapApp):
    
    @soapmethod(String,Integer,_returns=Array(String))
    def say_hello(self,name,times):
        results = []
        raise Exception("this is some crazy crap")
        for i in range(0,times):
            results.append('Hello, %s'%name)
        return results
    
    def onCall(self,environ):
        request.additional['call_start'] = time()

    def onMethodExec(self,environ,body,py_params,soap_params):
        request.additional['method_start'] = time()

    def onResults(self,environ,py_results,soap_results):
        request.additional['method_end'] = time()

    def onReturn(self,environ,returnString):
        call_start = request.additional['call_start']
        call_end = time()
        method_start = request.additional['method_start']
        method_end = request.additional['method_end']
        
        print 'Method took [%s] - total execution time[%s]'%(method_end-method_start,call_end-call_start)
        

def make_client():
    from soaplib.client import make_service_client
    client = make_service_client('http://localhost:7889/',HelloWorldService())
    return client
    
if __name__=='__main__':
    try:
        from wsgiref.simple_server import make_server
        server = make_server('localhost', 7889, HelloWorldService())
        server.serve_forever()
    except ImportError:
        print "Error: example server code requires Python >= 2.5"


import unittest
import datetime
from soaplib.etimport import ElementTree
from soaplib.xml import ns, create_xml_element
from soaplib.serializers.primitive import *

class test(unittest.TestCase):

    def test_string(self):
        s = String()
        element = String.to_xml('value')
        self.assertEquals(element.text,'value')
        value = String.from_xml(element)
        self.assertEquals(value,'value')
    
    def test_datetime(self):
        d = DateTime()
        n = datetime.datetime.now()
        element = DateTime.to_xml(n)
        self.assertEquals(element.text,n.isoformat())
        dt = DateTime.from_xml(element)
        self.assertEquals(n,dt)

    def test_utcdatetime(self):
        datestring = '2007-05-15T13:40:44Z'
        e = create_xml_element('test', ns)
        e.text = datestring
        
        dt = DateTime.from_xml(e)    

        self.assertEquals(dt.year,2007)
        self.assertEquals(dt.month,5)
        self.assertEquals(dt.day,15)
        
        datestring = '2007-05-15T13:40:44.003Z'
        e = create_xml_element('test', ns)
        e.text = datestring
        
        dt = DateTime.from_xml(e)    

        self.assertEquals(dt.year,2007)
        self.assertEquals(dt.month,5)
        self.assertEquals(dt.day,15)
        

    def test_integer(self):
        i = 12
        integer = Integer()
        element = Integer.to_xml(i)
        self.assertEquals(element.text,'12')
        self.assertEquals('xs:int', element.get(ns.get('xsi') + 'type'))
        value = integer.from_xml(element)
        self.assertEquals(value,i)

    def test_float(self):
        f = 1.22255645
        element = Float.to_xml(f)
        self.assertEquals(element.text,'1.22255645')
        self.assertEquals('xs:float', element.get(ns.get('xsi') + 'type'))
        f2 = Float.from_xml(element)
        self.assertEquals(f2,f)

    def test_array(self):
        serializer = Array(String)
        values = ['a','b','c','d','e','f']
        element = serializer.to_xml(values)
        self.assertEquals(len(values),len(element.getchildren()))
        values2 = serializer.from_xml(element)
        self.assertEquals(values[3],values2[3])

    def test_unicode(self):
        s = u'\x34\x55\x65\x34'         
        self.assertEquals(4,len(s))
        element = String.to_xml(s)
        value = String.from_xml(element)
        self.assertEquals(value,s)

    def test_null(self):
        element = Null.to_xml('doesnt matter')
        self.assertEquals('1',element.get(ns.get('xs') + 'null'))
        value = Null.from_xml(element)
        self.assertEquals(None,value)
        
    def test_boolean(self):
        b = Boolean.to_xml(True)
        self.assertEquals('true',b.text)

        b = Boolean.to_xml(0)
        self.assertEquals('false',b.text)

        b = Boolean.to_xml(1)
        self.assertEquals('true',b.text)

        b = Boolean.from_xml(b)
        self.assertEquals(b,True)
        
        b = Boolean.to_xml(False)
        self.assertEquals('false',b.text)
        
        b = Boolean.from_xml(b)
        self.assertEquals(b,False)

        b = Boolean.to_xml(False)
        self.assertEquals('xs:boolean', b.get(ns.get('xsi') + 'type'))

        b = Boolean.to_xml(None)
        self.assertEquals('1', b.get(ns.get('xs') + 'null'))
        
        b = Boolean.from_xml(b)
        self.assertEquals(b,None)
        
    def test_repeating(self):
        serializer = Repeating(String)
        
        data = ["a","b","c","d"]
        
        elements = serializer.to_xml(data)
        self.assertEquals(len(elements),4)
        
        newdata = serializer.from_xml(*elements)
        
        self.assertEquals(data,newdata)


def test_suite():
    loader = unittest.TestLoader()
    return loader.loadTestsFromTestCase(test)

if __name__== '__main__':
    unittest.TextTestRunner().run(test_suite())




import unittest

import primitive_test
import clazz_test
import binary_test

def test_suite():
    loader = unittest.TestLoader()
    suite = loader.loadTestsFromTestCase(primitive_test.test)
    suite.addTests(clazz_test.test_suite())
    suite.addTests(binary_test.test_suite())    
    return suite
    
if __name__== '__main__':
    unittest.TextTestRunner().run(test_suite())


import unittest
import datetime
from soaplib.xml import NamespaceLookup
from soaplib.serializers.primitive import *
from soaplib.serializers.clazz import *


##########################################################
# Simple Classes
##########################################################

class Address(ClassSerializer):
    class types:
        street = String
        city = String
        zip = Integer
        since = DateTime
        laditude = Float
        longitude = Float

class Person(ClassSerializer):
    class types:
        name = String
        birthdate = DateTime
        age = Integer
        addresses = Array(Address)
        titles = Array(String)
        
class ClassWithRepeatingData(ClassSerializer):
    class types:
        people = Repeating(Person)

##########################################################
# Complex Classes
##########################################################


class Level2(ClassSerializer):
    class types:
        arg1 = String
        arg2 = Float

class Level3(ClassSerializer):
    class types:
        arg1 = Integer

class Level4(ClassSerializer):
    class types:
        arg1 = String

class Level1(ClassSerializer):
    class types:
        level2 = Level2
        level3 = Array(Level3)
        level4 = Array(Level4)

 
        

class test(unittest.TestCase):

    def test_simple_class(self):
        a = Address()
        a.street = '123 happy way'
        a.city = 'badtown'
        a.zip = 32
        a.laditude = 4.3
        a.longitude = 88.0

        element = Address.to_xml(a)
        self.assertEquals(6,len(element.getchildren()))

        r = Address.from_xml(element)

        self.assertEquals(a.street,r.street)
        self.assertEquals(a.city,r.city)
        self.assertEquals(a.zip,r.zip)
        self.assertEquals(a.laditude,r.laditude)
        self.assertEquals(a.longitude,r.longitude)
        self.assertEquals(a.since,r.since)

    def test_nested_class(self):
        p = Person()
        element = Person.to_xml(p)
        
        self.assertEquals(None,p.name)
        self.assertEquals(None,p.birthdate)
        self.assertEquals(None,p.age)
        self.assertEquals(None,p.addresses)

        p2 = Person()
        
    def test_class_array(self):
    
        peeps = []
        names = ['bob','jim','peabody','mumblesleves']
        for name in names:
            a = Person()
            a.name = name
            a.birthdate = datetime.datetime(1979,1,1)
            a.age = 27
            peeps.append(a)

        serializer = Array(Person)
        element = serializer.to_xml(peeps)

        self.assertEquals(4,len(element.getchildren()))

        peeps2 = serializer.from_xml(element)   
        for i in range(0,4):
            self.assertEquals(peeps[i].name,names[i])
            self.assertEquals(peeps[i].birthdate,datetime.datetime(1979,1,1))

    def test_class_nested_array(self):
        peeps = []
        names = ['bob','jim','peabody','mumblesleves']
        for name in names:
            a = Person()
            a.name = name
            a.birthdate = datetime.datetime(1979,1,1)
            a.age = 27
            a.addresses = []
            
            for i in range(0,25):
                addr = Address()
                addr.street = '555 downtown'
                addr.city = 'funkytown'
                a.addresses.append(addr)

            peeps.append(a)


        serializer = Array(Person)
        element = serializer.to_xml(peeps)

        self.assertEquals(4,len(element.getchildren()))

        peeps2 = serializer.from_xml(element)
        for peep in peeps2:
            self.assertEquals(27,peep.age)
            self.assertEquals(25,len(peep.addresses))
            self.assertEquals('funkytown',peep.addresses[18].city)
            

    def test_complex_class(self):
        l = Level1()

        l.level2 = Level2()
        l.level2.arg1 = 'abcd'
        l.level2.arg2 = 1.444


        l.level3 = []
        l.level4 = []

        for i in range(0,100):
            a = Level3()
            a.arg1 = i
            l.level3.append(a)

        for i in range(0,4):
            a = Level4()
            a.arg1 = str(i)
            l.level4.append(a)

        element = Level1.to_xml(l)

        l1 = Level1.from_xml(element)

        self.assertEquals(l1.level2.arg1,l.level2.arg1)
        self.assertEquals(l1.level2.arg2,l.level2.arg2)
        self.assertEquals(len(l1.level4),len(l.level4))
        self.assertEquals(100,len(l.level3))

    def test_schema(self):
        a = {}
        Person.add_to_schema(a, NamespaceLookup())
        #self.assertEquals(8,len(a))
        self.assertTrue(a.has_key(ns.get('tns') + "Person"))
        self.assertTrue(a.has_key(ns.get('tns') + "Address"))
        self.assertTrue(a.has_key(ns.get('tns') + "AddressArray"))

    def test_repeating(self):
        
        peeps = []
        names = ['bob','jim','peabody','mumblesleves']
        for name in names:
            a = Person()
            a.name = name
            a.birthdate = datetime.datetime(1979,1,1)
            a.age = 27
            a.addresses = []
            
            for i in range(0,25):
                addr = Address()
                addr.street = '555 downtown'
                addr.city = 'funkytown'
                a.addresses.append(addr)

            peeps.append(a)
            
        rpt = ClassWithRepeatingData()
        rpt.people = peeps
        
        e = ClassWithRepeatingData.to_xml(rpt)
        rpt2 = ClassWithRepeatingData.from_xml(e)
        
        self.assertEquals(len(rpt2.people),len(rpt.people))
        

def test_suite():
    loader = unittest.TestLoader()
    return loader.loadTestsFromTestCase(test)

if __name__== '__main__':
    unittest.TextTestRunner().run(test_suite())

      


from soaplib.wsgi_soap import SimpleWSGISoapApp
from soaplib.service import soapmethod
from soaplib.serializers.primitive import String, Integer, Array
from soaplib.serializers.binary import Attachment
from soaplib.util import get_callback_info

from threading import Thread
from tempfile import mkstemp
import time

'''
This is a very simple async service that sleeps for a specified 
number of seconds and then call back the caller with a message.
This kicks off a new Thread for each request, which is not recommended
for a real-world application.  Soaplib does not provide any thread
management or scheduling mechanism, the service is responsible for the
execution of the async. process.
'''

class SleepingService(SimpleWSGISoapApp):
    
    @soapmethod(Integer,_isAsync=True)
    def sleep(self,seconds):
        msgid, replyto = get_callback_info()
        
        def run():
            time.sleep(seconds)
            
            client = create_service_client(replyto, self)
            client.woke_up('good morning',msgid=msgid)

        Thread(target=run).start()

    @soapmethod(String,_isCallback=True)
    def woke_up(self,message):
        pass
        
if __name__=='__main__':
    try:
        from wsgiref.simple_server import make_server
        server = make_server('localhost', 7789, SleepingService())
        server.serve_forever()
    except ImportError:
        print "Error: example server code requires Python >= 2.5"


import unittest
import shutil
import base64
from soaplib.serializers.binary import *
from soaplib.xml import NamespaceLookup
from tempfile import mkstemp
import os, shutil


class test(unittest.TestCase):

    def setUp(self):
        os.mkdir('binaryDir')
        
        fd, self.tmpfile = mkstemp('','','binaryDir')
        os.close(fd)
        f = open(self.tmpfile,'w')
        for i in range(0,1000):
            f.write('All work and no play makes jack a dull boy\r\n')
        f.flush()
        f.close()

    def tearDown(self):
        shutil.rmtree('binaryDir')

    def test_to_xml_data(self):
        f = open(self.tmpfile)
        data = f.read()
        f.close()
    
        a = Attachment()
        a.data = data
        element = Attachment.to_xml(a)

        encoded_data = base64.encodestring(data)

        self.assertNotEquals(element.text,None)
        self.assertEquals(element.text,encoded_data)

    def test_to_xml_file(self):
        a = Attachment()
        a.fileName = self.tmpfile
        
        f = open(self.tmpfile,'rb')
        data = f.read()
        f.close()

        element = Attachment.to_xml(a)
        encoded_data = base64.encodestring(data)

        self.assertNotEquals(element.text,None)
        self.assertEquals(element.text,encoded_data)

    def test_to_from_xml_file(self):
        a = Attachment()
        a.fileName = self.tmpfile


        element = Attachment.to_xml(a)
        data = Attachment.from_xml(element).data


        f = open(self.tmpfile,'rb')
        fdata = f.read()
        f.close()

        self.assertEquals(data, fdata)


    def test_exception(self):
        try:
            Attachment.to_xml(Attachment())
        except:
            self.assertTrue(True)
        else:
            self.assertFalse(True)

    def test_from_xml(self):
        f = open(self.tmpfile)
        data = f.read()
        f.close()
    
        a = Attachment()
        a.data = data
        element = Attachment.to_xml(a)

        a2 = Attachment.from_xml(element)
        self.assertEquals(data,a2.data)

    def test_add_to_schema(self):
        schema = {}
        Attachment.add_to_schema(schema, NamespaceLookup())
        self.assertEquals(0,len(schema.keys()))

    def test_get_datatype(self):
        dt = Attachment.get_datatype()
        self.assertEquals('base64Binary', dt)
        dt = Attachment.get_datatype(ns)

        self.assertEquals(ns.get('xs') + 'base64Binary', dt)        


def test_suite():
    loader = unittest.TestLoader()
    return loader.loadTestsFromTestCase(test)

if __name__== '__main__':
    unittest.TextTestRunner().run(test_suite())


        



from soaplib.wsgi_soap import SimpleWSGISoapApp
from soaplib.service import soapmethod
from soaplib.serializers.primitive import String, Integer, Array

'''
This example shows how to override the variable names for fun and profit.
This is very useful for situations that require the use of variable names
that are python keywords like, from, to, import, return, etc.
'''

class EmailManager(SimpleWSGISoapApp):
    
    @soapmethod(String,String,String,
                _inVariableNames={'_to':'to','_from':'from','_message':'message'},
                _outVariableName='return')    
    def sendEmail(self,_to,_from,message):
        # do email sending here
        return 'sent!'

if __name__=='__main__':
    try:
        from wsgiref.simple_server import make_server
        server = make_server('localhost', 7789, EmailManager())
        server.serve_forever()
    except ImportError:
        print "Error: example server code requires Python >= 2.5"



import unittest
import datetime
from soaplib.etimport import ElementTree

from soaplib.serializers.primitive import *
from soaplib.serializers.clazz import *
from soaplib.service import *
from soaplib.wsgi_soap import *
from soaplib.client import *
from soaplib.util import *
from soaplib.soap import *

from threading import Thread
try:
    from wsgiref.simple_server import make_server
except ImportError:
    raise Exception("UnitTests require Python >= 2.5")

class Address(ClassSerializer):
    class types:
        street = String
        city = String
        zip = Integer
        since = DateTime
        laditude = Float
        longitude = Float

class Person(ClassSerializer):
    class types:
        name = String
        birthdate = DateTime
        age = Integer
        addresses = Array(Address)
        titles = Array(String)

class Request(ClassSerializer):
    class types:
        param1 = String
        param2 = Integer

class Response(ClassSerializer):
    class types:
        param1 = Float
      
class TestService(SimpleWSGISoapApp):

    @soapmethod(String, Integer, _returns=DateTime)
    def a(self, s, i):
        return datetime.datetime(1901,12,15)

    @soapmethod(Person, String, Integer, _returns=Address)
    def b(self, p,s,i):
        a = Address()
        a.zip = 4444
        a.street = 'wsgi way'
        a.laditude = 123.3
        
        return a
        
    @soapmethod(Person, _isAsync=True)
    def d(self, person):
        pass

    @soapmethod(Person, _isCallback=True)
    def e(self, person):
        pass
        
    @soapmethod()
    def fault(self):
        raise Exception('Testing faults')

class test(unittest.TestCase):

    def setUp(self):
        self.server = make_server('127.0.0.1', 9191, TestService())
        self.server.allow_reuse_address = True
        Thread(target=self.server.serve_forever).start()

    def tearDown(self):
        self.server.shutdown()
        del self.server

    def test_simple(self):
        inMessage = Message('a',[('s',String),('i',Integer)])
        outMessage = Message('aResponse',[('retval',DateTime)])
        
        desc = MethodDescriptor('a','a',inMessage,outMessage,'')

        client = SimpleSoapClient('127.0.0.1:9191','/',desc)
        results = client('abc',54)
        self.assertEquals(results,datetime.datetime(1901,12,15))    

    def test_nested(self):
        inMessage = Message('b',[('p',Person),('s',String),('i',Integer)])
        outMessage = Message('bResponse',[('retval',Address)])
        
        desc = MethodDescriptor('b','b',inMessage,outMessage,'')

        client = SimpleSoapClient('127.0.0.1:9191','/',desc)
        p = Person()
        p.name = 'wilson'
        p.addresses = []
        for i in range(0,123):
            a = Address()
            a.zip = i
            p.addresses.append(a)
        res = client(p,'abc',123)
        self.assertEquals(res.longitude,None)
        self.assertEquals(res.zip,4444)
        self.assertEquals(res.street,'wsgi way')

    def test_async(self):
        inMessage = Message('d',[('person',Person)])
        outMessage = Message('dResponse',[])

        desc = MethodDescriptor('d','d',inMessage,outMessage,'')
        
        client = SimpleSoapClient('127.0.0.1:9191','/',desc)
        p = Person()
        p.name = 'wilson'
        r = client(p)
        self.assertEquals(r,None)
        
    def test_fault(self):
        inMessage = Message('fault',[])
        outMessage = Message('faultResponse',[])
        desc = MethodDescriptor('fault','fault',inMessage,outMessage,'')
        
        client = SimpleSoapClient('127.0.0.1:9191','/',desc)
        try:
            client()
        except Fault, f:
            self.assertEquals(f.faultcode,'faultFault')
            self.assertEquals(f.faultstring,'Testing faults')
            self.assertTrue(f.detail.find('client_test.py') > -1)
        else:
            raise 

    def _test_callback(self):
        inputs = [ParameterDescriptor('person',Person)]
        
        client = SimpleSoapClient('127.0.0.1:9191','/','e',inputs,None)
        p = Person()
        p.name = 'wilson'
        r = client(p)
        self.assertEquals(r,None)

    def test_service_client(self):        
        client = ServiceClient('127.0.0.1:9191','/',TestService())

        r = client.a('bobo',23)
        self.assertEquals(r,datetime.datetime(1901, 12, 15))    

        p = Person()
        p.name = 'wilson'
        p.addresses = []
        for i in range(0,123):
            a = Address()
            a.zip = i
            p.addresses.append(a)
        res = client.b(p,'abc',123)
        self.assertEquals(res.longitude,None)
        self.assertEquals(res.zip,4444)
        self.assertEquals(res.street,'wsgi way')

        request = Request()
        request.param1 = 'asdf'

        p = Person()
        p.name = 'wilson'
        r = client.d(p)
        self.assertEquals(r,None)

        p = Person()
        p.name = 'wilson'
        r = client.e(p)
        self.assertEquals(r,None)

def test_suite():
    #debug(True)
    loader = unittest.TestLoader()
    #log_debug(True)
    return loader.loadTestsFromTestCase(test)

if __name__== '__main__':
    unittest.TextTestRunner().run(test_suite())



    



#!/usr/bin/env python

from soaplib.client import make_service_client
from soaplib.serializers.binary import Attachment
from helloworld_attach import HelloWorldService
from soaplib.client import debug
debug(True)

client = make_service_client('http://localhost:7789/',HelloWorldService())
print client.say_hello(Attachment(data="Dave"),5,mtom=True)



#!/usr/bin/env python

from setuptools import setup, find_packages
import sys

VERSION = '0.8.1'
LONG_DESC = """\
This is a simple, easily extendible soap library that provides several useful
tools for creating and publishing soap web services in python.  This package
features on-demand wsdl generation for the published services, a
wsgi-compliant web application, support for complex class structures, binary
attachments, simple framework for creating additional serialization mechanisms
and a client library.

This prokect now uses lxml as it's XML API, providing full namespace support.
"""

setup(name='soaplib',
      version=VERSION,
      description="A simple library for writing soap web services",
      long_description=LONG_DESC,
      classifiers=[
          'Programming Language :: Python',
          'Operating System :: OS Independent',
          'Natural Language :: English',
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
      ],
      keywords='soap',
      author='Aaron Bickell',
      author_email='abickell@optio.com',
      maintainer = 'Jamie Kirkpatrick',
      maintainer_email = 'jkp@kirkconsulting.co.uk',
      url='http://wiki.github.com/jkp/soaplib-lxml',
      license='LGPL',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      zip_safe=False,
      install_requires=[
        'pytz',
        'lxml>=2.2.1',
      ],
      test_suite='tests.test_suite',
      entry_points="""
      """,
      )



from soaplib.service import soapmethod
from soaplib.wsgi_soap import SimpleWSGISoapApp, log_debug, log_exceptions
from soaplib.serializers.primitive import String, Integer, DateTime, Float, Boolean, Array, Any
from soaplib.serializers.clazz import ClassSerializer
from soaplib.serializers.binary import Attachment

class SimpleClass(ClassSerializer):
    class types:
        i = Integer
        s = String

class OtherClass(ClassSerializer):
    class types:
        dt = DateTime
        f = Float
        b = Boolean

class NestedClass(ClassSerializer):
    class types:
        simple = Array(SimpleClass)
        s = String
        i = Integer
        f = Float
        other = OtherClass

class InteropService(SimpleWSGISoapApp):

    # basic primitives
    @soapmethod(Integer, _returns=Integer)
    def echoInteger(self, i):
        return i
        
    @soapmethod(String, _returns=String)
    def echoString(self,s):
        return s
        
    @soapmethod(DateTime, _returns=DateTime)
    def echoDateTime(self,dt):
        return dt
    
    @soapmethod(Float,_returns=Float)
    def echoFloat(self,f):
        return f
        
    @soapmethod(Boolean,_returns=Boolean)
    def echoBoolean(self,b):
        return b
                
    # lists of primitives
    @soapmethod(Array(Integer), _returns=Array(Integer))
    def echoIntegerArray(self,ia):
        return ia
       
    @soapmethod(Array(String), _returns=Array(String))
    def echoStringArray(self, sa):
        return sa
        
    @soapmethod(Array(DateTime), _returns=Array(DateTime))
    def echoDateTimeArray(self,dta):
        return dta

    @soapmethod(Array(Float),_returns=Array(Float))
    def echoFloatArray(self,fa):
        return fa

    @soapmethod(Array(Boolean),_returns=Array(Boolean))
    def echoBooleanArray(self,ba):
        return ba
        
    # classses
    @soapmethod(SimpleClass,_returns=SimpleClass)
    def echoSimpleClass(self,sc):
        return sc
        
    @soapmethod(Array(SimpleClass),_returns=Array(SimpleClass))
    def echoSimpleClassArray(self,sca):
        return sca
        
    @soapmethod(NestedClass,_returns=NestedClass)
    def echoNestedClass(self,nc):
        return nc
        
    @soapmethod(Array(NestedClass),_returns=Array(NestedClass))
    def echoNestedClassArray(self,nca):
        return nca
        
    @soapmethod(Attachment,_returns=Attachment)
    def echoAttachment(self,a):
        return a

    @soapmethod(Array(Attachment),_returns=Array(Attachment))
    def echoAttachmentArray(self,aa):
        return aa
        
    @soapmethod()
    def testEmpty(self):
        # new
        pass
        
    @soapmethod(String,Integer,DateTime)
    def multiParam(self,s,i,dt):
        # new
        pass
        
    @soapmethod(_returns=String)
    def returnOnly(self):
        # new
        return 'howdy'
        
    @soapmethod(String,_returns=String,_soapAction="http://sample.org/webservices/doSomething")
    def doSomethingElse(self,s):
        return s
        
if __name__ == '__main__':
    try:
        from wsgiref.simple_server import make_server
        log_debug(True)
        log_exceptions(True)
        server = make_server('127.0.0.1',9753, InteropService())
        print 'Starting interop server at -- %s:%s' % ('127.0.0.1',9753)
        server.serve_forever()
    except ImportError:
        print "Error: example server code requires Python >= 2.5"

